#
#  AppController.rb
#  Clip Info Titler
#
#  Created by Gregory Clarke on 10/4/10.
#  Copyright (c) 2010 Intelligent Assistance, Inc. All rights reserved.
#

require 'osx/cocoa'
require 'clip_info_titles.bundle'

class AppController < OSX::NSObject
	
	ib_outlets :mainWindow, :deactivateMenuItem
	ib_outlets :citView, :spinner, :nameField, :frameRatePopup, :xmlIcon, :xmlFileName, :tcrCheckbox, :templatePopup, :line1Label, :line1Popup, :line2Label, :line2Popup, :line3Label, :line3Popup, :line4Label, :line4Popup, :line5Label, :line5Popup, :sendToFCPButton
	ib_outlets :regoView, :deactivateAreaView, :webView, :problemMessage, :closeRegoButton
	
	def initialize
		default_values = {"Line1Popup" => " ", "Line2Popup" => " ", "Line3Label" => "File Name:", "Line3Popup" => "File Name", "Line4Label" => "Reel Name:", "Line4Popup" => "Reel Name", "Line5Popup" => "Date (March 13, 2010), Time"}
		defaultsController = OSX::NSUserDefaultsController.sharedUserDefaultsController
		defaultsController.setInitialValues(default_values)
	end
	
	def awakeFromNib
		puts "awakeFromNib"
		check_for_expired_beta()
		@xmlIcon.setHidden(true)
		@xmlIcon.setToolTip("")
		@xmlFileName.setStringValue("")
	end

	### rego methods ###
	
	def applicationWillFinishLaunching(sender) # applicationWillFinishLaunching: is sent before application:openFile: (so it can deal with a file dropped onto the app icon)
		puts "applicationWillFinishLaunching"
		check_for_expired_beta()
		if @this_rego[0]["active"]
			showMainView()
		elsif @this_rego[0]["mac_problem"]==true
			check_for_rego(@this_rego)
		else
			@deactivateMenuItem.setTitle("Activate...")
			displayDemoSheet()
		end
	end
	
	# def applicationDidBecomeActive(notification)
	# 	puts "applicationDidBecomeActive"
	# end
	
	def displayDemoSheet
		alert = OSX::NSAlert.alloc.init
		alert.setMessageText("#{SKU} has not been activated for this computer.")
		alert.setAlertStyle(OSX::NSInformationalAlertStyle)
		alert.setInformativeText("Would you like to activate #{SKU}, or try it in demo mode (which is limited to random clips)?")
		alert.addButtonWithTitle("Activate")
		alert.addButtonWithTitle("Buy")
		alert.addButtonWithTitle("Demo")
		alert.beginSheetModalForWindow_modalDelegate_didEndSelector_contextInfo(@mainWindow, self, "didEndDemoSheet:returnCode:contextInfo:", nil)
	end
	
	def didEndDemoSheet_returnCode_contextInfo(sheet, buttonClicked, info)
		sheet.window.orderOut(nil)
		if buttonClicked==OSX::NSAlertSecondButtonReturn
			system('open', 'http://www.AssistedEditing.com/ClipInfoTitler/buy')
		end
		if buttonClicked==OSX::NSAlertThirdButtonReturn
			showMainView()
		else
			newFrame = self.newFrameForNewContentView(@regoView)
			showActivationView(newFrame, WEB_APP + "license/" + SKU.gsub(" ", "%20") + "?sn=" + @this_rego[0]["number"].to_s + "&mac=" + MAC) # NEW
		end
	end
	
	def showDeactivation
		if @deactivateMenuItem.title=="Activate..."
			newFrame = self.newFrameForNewContentView(@regoView)
			showActivationView(newFrame, WEB_APP + "license/" + SKU.gsub(" ", "%20") + "?sn=" + @this_rego[0]["number"].to_s + "&mac=" + MAC) # NEW
		else
			newFrame = self.newFrameForNewContentView(@deactivateAreaView) # 500 x 200
			@mainWindow.setMinSize(OSX::NSSize.new(500, 200+22))
			@mainWindow.setMaxSize(OSX::NSSize.new(500, 200+22))
			@mainWindow.setContentView(@deactivateAreaView)
			showActivationView(newFrame, WEB_APP + "destroy/" + @this_rego[0]["number"].to_s)
		end
		@closeRegoButton.setTitle("Cancel")
	end
	ib_action :showDeactivation # Clip Info Titler > Activate or Deactivate
	
	def showActivationView(frame, url)
		puts "showActivationView #{url}"
		@mainWindow.setFrame_display frame, true
		@mainWindow.setContentView(@regoView)
		@mainWindow.setTitle("Activation")
		# notification
		center = OSX::NSNotificationCenter.defaultCenter
		center.addObserver_selector_name_object(self, :webViewProgressFinished, "WebProgressFinishedNotification", @webView) # name is WebProgressFinishedNotification NOT WebViewProgressFinishedNotification
		@webView.mainFrame.loadRequest( OSX::NSURLRequest.requestWithURL( OSX::NSURL.URLWithString(url) ) )
	end
	
	def webViewProgressFinished(notification)
		puts notification.name
		if notification.name=="WebProgressFinishedNotification"
			if @webView.mainFrame.dataSource.nil? # in case page doesn't load...
				@webView.mainFrame.stopLoading
				@problemMessage.setHidden(false)
				@webView.setHidden(true)
			else
				get_web_app_response(@webView.mainFrame.dataSource.response.URL)
			end
		end
	end 
	
	def closeActivationView
		if @closeRegoButton.title=="Continue" || @closeRegoButton.title=="Cancel"
			showMainView()
		else
			OSX::NSApplication.sharedApplication.terminate(self)
		end
	end
	ib_action :closeActivationView # on click of Registration window's Continue button
	
	### end rego methods ###
	
	def showMainView
		setupView(@citView, "Clip Info Titler", 500, 554)
	end
	
	def setupView(view, title, width, height)
		@mainWindow.setTitle(title)
		oldFrame = @mainWindow.frame
		newFrame = self.newFrameForNewContentView(view)
		@mainWindow.setMinSize(OSX::NSSize.new(width, height+22))
		@mainWindow.setFrame_display_animate(newFrame, true, true)
		@mainWindow.setContentView(view)
	end
	
	def newFrameForNewContentView(view)
		newFrameRect = @mainWindow.frameRectForContentRect(view.frame)
		oldFrameRect = @mainWindow.frame
		newSize = newFrameRect.size
		oldSize = oldFrameRect.size
		frame = @mainWindow.frame
		frame.size = newSize
		frame.origin.y = frame.origin.y - (newSize.height - oldSize.height)
		frame
	end

	def showHelp(sender)
		openPDF("Clip Info Titler - Help.pdf")
	end
	ib_action :showHelp # Help > Clip Info Titler Help
	
	def openPDF(filename)
		path_to_file = __FILE__.sub("AppController.rb",filename)
		system('open', path_to_file)
	end

	def visitWebsite(sender)
		system('open', 'http://www.AssistedEditing.com/')
	end
	ib_action :visitWebsite # Help > Visit www.AssistedEditing.com

	def sendFeedback(sender)
		OSX::JRFeedbackController.showFeedback
	end
	ib_action :sendFeedback # Clip Info Titler > Provide Feedback

	def sendSupportEmail(subject)
		system('open', "mailto:techsupport@intelligentassistance.com?subject=Clip Info Titler #{subject}&body=Attached is the XML file that Clip Info Titler was unable to process.")
	end
	
	def chooseXmlFile(sender)
		filetypes = ["xml"]
		oPanel = OSX::NSOpenPanel.openPanel
		oPanel.setTitle("Choose the exported Bin(s) of Clips to process (XML file)")
		oPanel.setAllowsMultipleSelection(false)
		oPanel.beginSheetForDirectory_file_types_modalForWindow_modalDelegate_didEndSelector_contextInfo(nil, nil, filetypes, @mainWindow, self, 'didEndOpenSheet:returnCode:contextInfo:', nil)
	end
	ib_action :chooseXmlFile

	def didEndOpenSheet_returnCode_contextInfo(oPanel, buttonClicked, info)
		if buttonClicked == OSX::NSOKButton
			oPanel.orderOut(self)
			path = oPanel.filenames.objectAtIndex(0)
			application_openFile(self, path)
		end
	end

	def application_openFile(sender, path)
		OSX::NSDocumentController.sharedDocumentController().noteNewRecentDocumentURL( OSX::NSURL.fileURLWithPath(path) ) # add this file to the Recent menu
		# fileManager = OSX::NSFileManager.defaultManager
		chosen_file = path.to_s
		chosen_file_extension = File.extname(chosen_file)
		begin
			@spinner.startAnimation(self)
			get_info_from_clips(chosen_file)
			@spinner.stopAnimation(self)
			@sendToFCPButton.setEnabled(true)
			# Sequence Settings box
			@frameRatePopup.removeAllItems
			@frameRatePopup.addItemsWithTitles(@frame_rates)
			@frameRatePopup.setEnabled(true)
			# Titles box
			@templatePopup.removeAllItems
			@templates_array.each do |item|
				@templatePopup.addItemWithTitle(item[:name])
			end
			# Line popup menus
			times = @clips_array.collect { |c| c[:time] }.compact
			puts times.inspect
			if times.empty? # all :time were nil
				puts "disabling date items"
				@line1Popup.itemAtIndex(9).setEnabled(false)
				@line1Popup.itemAtIndex(10).setEnabled(false)
				@line1Popup.itemAtIndex(11).setEnabled(false)
				@line1Popup.itemAtIndex(12).setEnabled(false)
				@line2Popup.itemAtIndex(9).setEnabled(false)
				@line2Popup.itemAtIndex(10).setEnabled(false)
				@line2Popup.itemAtIndex(11).setEnabled(false)
				@line2Popup.itemAtIndex(12).setEnabled(false)
				@line3Popup.itemAtIndex(9).setEnabled(false)
				@line3Popup.itemAtIndex(10).setEnabled(false)
				@line3Popup.itemAtIndex(11).setEnabled(false)
				@line3Popup.itemAtIndex(12).setEnabled(false)
				@line4Popup.itemAtIndex(9).setEnabled(false)
				@line4Popup.itemAtIndex(10).setEnabled(false)
				@line4Popup.itemAtIndex(11).setEnabled(false)
				@line4Popup.itemAtIndex(12).setEnabled(false)
				@line5Popup.itemAtIndex(9).setEnabled(false)
				@line5Popup.itemAtIndex(10).setEnabled(false)
				@line5Popup.itemAtIndex(11).setEnabled(false)
				@line5Popup.itemAtIndex(12).setEnabled(false)
			else
				@line1Popup.itemAtIndex(9).setEnabled(true)
				@line1Popup.itemAtIndex(10).setEnabled(true)
				@line1Popup.itemAtIndex(11).setEnabled(true)
				@line1Popup.itemAtIndex(12).setEnabled(true)
				@line2Popup.itemAtIndex(9).setEnabled(true)
				@line2Popup.itemAtIndex(10).setEnabled(true)
				@line2Popup.itemAtIndex(11).setEnabled(true)
				@line2Popup.itemAtIndex(12).setEnabled(true)
				@line3Popup.itemAtIndex(9).setEnabled(true)
				@line3Popup.itemAtIndex(10).setEnabled(true)
				@line3Popup.itemAtIndex(11).setEnabled(true)
				@line3Popup.itemAtIndex(12).setEnabled(true)
				@line4Popup.itemAtIndex(9).setEnabled(true)
				@line4Popup.itemAtIndex(10).setEnabled(true)
				@line4Popup.itemAtIndex(11).setEnabled(true)
				@line4Popup.itemAtIndex(12).setEnabled(true)
				@line5Popup.itemAtIndex(9).setEnabled(true)
				@line5Popup.itemAtIndex(10).setEnabled(true)
				@line5Popup.itemAtIndex(11).setEnabled(true)
				@line5Popup.itemAtIndex(12).setEnabled(true)
			end
		rescue
			@spinner.stopAnimation(self)
			alert = OSX::NSAlert.alloc.init
			alert.setMessageText("There was a problem reading the XML file.")
			alert.setAlertStyle(OSX::NSInformationalAlertStyle)
			puts "RESCUE!!! Problem with get_info_from_clips()"
			alert.setInformativeText("Please choose another XML file exported from\nFinal Cut Pro.\n\nWould you like to email this XML file to Tech Support?")
			alert.addButtonWithTitle("Email Tech Support")
			alert.addButtonWithTitle("Ignore")
			buttonClicked = alert.runModal
			sendSupportEmail("problem reading XML file") if buttonClicked==OSX::NSAlertFirstButtonReturn
		end
	end

	def send_to_fcp
		begin
			@spinner.startAnimation(self)
			chosen_frame_rate = FRAME_RATES.find { |rate| rate[:type] == @frameRatePopup.titleOfSelectedItem }
			@ntsc = chosen_frame_rate[:ntsc]
			@timebase = chosen_frame_rate[:timebase]
			create_fcp_xml()
			# OSX::NSWorkspace.sharedWorkspace.openFile_withApplication( TEMP_XML_FILE, "Final Cut Pro" )
			fcp_path = OSX::NSWorkspace.sharedWorkspace.absolutePathForAppBundleWithIdentifier("com.apple.FinalCutPro")
			OSX::NSWorkspace.sharedWorkspace.openFile_withApplication(TEMP_XML_FILE, fcp_path) unless fcp_path.nil?
			@spinner.stopAnimation(self)
		rescue
			@spinner.stopAnimation(self)
			alert = OSX::NSAlert.alloc.init
			alert.setMessageText("There was a problem sending to Final Cut Pro.")
			alert.setAlertStyle(OSX::NSInformationalAlertStyle)
			alert.setInformativeText("Please choose another XML file exported from\nFinal Cut Pro.")
			alert.runModal
		end
	end

	def applicationShouldTerminateAfterLastWindowClosed(sender)
		true
	end

end
